// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  crystal growth group                                                  ==
// ==                                                                        ==
// ==  Stiftung caesar                                                       ==
// ==  Ludwig-Erhard-Allee 2                                                 ==
// ==  53175 Bonn                                                            ==
// ==  germany                                                               ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  http://www.caesar.de/cg/AMDiS                                         ==
// ==                                                                        ==
// ============================================================================

/** \file GlobalElementNumbering.h */

#ifndef AMDIS_GLOBALELEMENTNUMBERING_H
#define AMDIS_GLOBALELEMENTNUMBERING_H

// ===========================================================================
// ===== includes ============================================================
// ===========================================================================
#include <map>

namespace AMDiS {

  // =========================================================================
  // ===== forward declarations ==============================================
  // =========================================================================
  class MeshStructure;
  class Mesh;

  // =========================================================================
  // ===== class GlobalElementNumbering ======================================
  // =========================================================================

  /** \ingroup Parallel
   * \brief 
   * Creates a global element numbering.
   */
  class GlobalElementNumbering
  {
  public:
    /** \brief
     * Constructor
     */
    GlobalElementNumbering(MeshStructure *compositeStructure,
			   Mesh *localMesh);

    /** \brief
     * Returns the local index corresponding to local index.
     */
    int getLocalIndex(int globalIndex);

    /** \brief
     * Returns the global index corresponing to local index.
     */
    int getGlobalIndex(int localIndex);

  protected:
    /** \brief
     * Maps each local index to its global index.
     */
    ::std::map<int, int> localToGlobal_;

    /** \brief
     * Stores local index for each global index.
     */
    ::std::map<int, int> globalToLocal_;
  };

}

#endif
