#include "AMDiS.h"

using namespace std;
using namespace AMDiS;

// ===========================================================================
// ===== function definitions ================================================
// ===========================================================================

/** \brief
 * RHS function
 */
class F : public AbstractFunction<double, WorldVector<double> >
{
public:
  MEMORY_MANAGED(F);

  F(int degree) : AbstractFunction<double, WorldVector<double> >(degree) {};

  /** \brief
   * Implementation of AbstractFunction::operator().
   */
  const double& operator()(const WorldVector<double>& x) const {
    static double result = 0.0;
    result = -2 * x[0];
    return result;
  };
};

// ===========================================================================
// ===== main program ========================================================
// ===========================================================================

int main(int argc, char* argv[])
{
  FUNCNAME("sphere main");

  // ===== check for init file =====
  TEST_EXIT(argc == 2)("usage: sphere initfile\n");

  // ===== init parameters =====
  Parameters::init(false, argv[1]);

  // ===== create projection =====
  WorldVector<double> ballCenter;
  ballCenter.set(0.0);
  NEW BallProject(1, 
		  BOUNDARY_PROJECTION, 
		  ballCenter, 
		  1.0);

  // ===== create and init the scalar problem ===== 
  ProblemScal sphere("sphere");
  sphere.initialize(INIT_ALL);

  // === create adapt info ===
  AdaptInfo *adaptInfo = NEW AdaptInfo("sphere->adapt");

  // === create adapt ===
  AdaptStationary *adapt = NEW AdaptStationary("sphere->adapt",
					       &sphere,
					       adaptInfo);
  
  // ===== create matrix operator =====
  Operator matrixOperator(Operator::MATRIX_OPERATOR,
			  sphere.getFESpace());
  matrixOperator.addSecondOrderTerm(NEW Laplace_SOT);
  sphere.addMatrixOperator(&matrixOperator);

  // ===== create rhs operator =====
  Operator rhsOperator(Operator::VECTOR_OPERATOR,
		       sphere.getFESpace());

  int degree = sphere.getFESpace()->getBasisFcts()->getDegree();

  rhsOperator.addZeroOrderTerm(NEW CoordsAtQP_ZOT(NEW F(degree)));
  sphere.addVectorOperator(&rhsOperator);

  // ===== start adaption loop =====
  adapt->adapt();

  sphere.writeFiles(adaptInfo, true);

  return 0;
}


